/********************************************************************
*  This is a header file for the SORCUS libraries (all platforms,
*  all boards). It defines the channel property structures (CPS) 
*  and constants for the MODULAR-Device-Drivers (MDD).
*
*  Defines for Preprocessor:
*
*  __RT__         Must be done exclusive for real-time-projects
*                 (is done by header MLxRTBIB.H)
*
*  Author         Dipl.-Ing. H.Doersam
*
*  Last change    See HISTORY
*
*  (C) SORCUS Computer GmbH 1997 .. 1998
*      All rights reserved
********************************************************************/


/*                         

   +++++ IMPORTANT +++++

   This file defines some data structures that must be included with
   BYTE alignment. Use the pragma directive before including this
   file.

*/



/********************************************************************
*
*  HISTORY
*
*  08/05/98 BUILD 0x62050804 (HD)
*     -  New constants
*        ERR_MDD_WRONG_OSX
*        _CP_NO_FLAGS
*        _MC163_NO_TIME
*     -  BUG: Wrong defines for constants _MC163_FREEZE_DIN_...
*
*  27/04/98 BUILD 0x62041B01 (HD)
*     -  New constant
*        CDT_ID_CHANNEL_VALID
*
*  17/04/98 BUILD 0x62041105 (HD)
*     -  Support of MDD for module M-AX-32/SSI
*     -  Support of MDD for 5B panels
*
*  09/04/98 BUILD 0x62040904 (HD)
*     -  New constant
*        DEVICE_SSI
*
*  18/03/98 BUILD 0x62031203 (HD)
*     -  Support of MDD for Multi-LAB/2 board
*     -  Error in constants for M-DC15-2 
*        (.._IPx_... replaced by .._IP_x_...)
*
*  11/03/98 BUILD 0x62030B03 (HD)
*     -  All CPS structure redefined
*
*  09/03/98 BUILD 0x62030901 (HD)
*     -  Support of MDD for module M-AD16-3
*
*  03/03/98 BUILD 0x62030302 (HD)
*     -  Support of MDD for 5Bx64 panels
*     -  New constants for M-D40-2
*     -  New constants
*        DSC_SHARED_MAX
*        DSC_IS_NOT_AVAILABLE
*        DSC_IS_LOCKED
* 
*  26/02/98 BUILD 0x62021A04 (HD)
*     -  New constants 
*        ML8_WATCHDOG_x
*        DIAG_DATA_LOST
*     -  BUGFIX: Wrong values for constants INFO_x corrected
* 
*  17/02/98 BUILD 0x62021102 (HD)
*     -  New constants DIAG_xxx (diagnosis words)
* 
*  11/02/98 BUILD 0x62020B03 (HD)
*     -  New constants for M-SH12-8
*
*  10/02/98 BUILD 0x62020A02 (HD)
*     -  File consists all constants for the MDDs
*
*  09/02/98 BUILD 0x62020901 (HD)
*     -  Support of MDD for following modules:
*        M-DC15-2, M-C16-3, ML8 (Base Board)
*     -  Redefinition of CPS CPS_MSH128
*     -  Redefinition of CPS CPS_MOPT1B
*     -  Redefinition of CPS CPS_MRU82
*
*  25/09/97 BUILD 0x61091904 (HD)
*     -  Creation of this file
*     -  Support of MDDs fir following modules:
*        M-D40-2, M-OPT-1/A, M-OPT-1/B, M-RU8-2, M-AD16-4, M-DA16-2,
*        M-DA2-2, M-DA4-2, M-AD12-16, M-SH12-8, M-5B-1/U
*
********************************************************************/



#ifndef _INC_MDDCPS
#define _INC_MDDCPS



// Version of this file (DATE CODE of Build day)

#define BUILD_MDDCPS       0x62050804UL



// Commands for mdd8_send_channel_command

#define CMD_CLOSE          0x00A0   // Close the channel
                                    // HINT: Do not use this 
                                    // command,use procedure
                                    // mddX_close_channel
#define CMD_SUSP_WRITE     0x00A1   // Lock data output service
#define CMD_CONT_WRITE     0x00A2   // Unlock data outpuit service
#define CMD_SUSP_READ      0x00A3   // Lock data input service
#define CMD_CONT_READ      0x00A4   // Unlock data input service
#define CMD_RESET          0x00A5   // Reset the channel
#define CMD_START          0x00A6   // Start the channel
#define CMD_STOP           0x00A7   // Stop the channel
#define CMD_CONTINUE       0x00A8   // Continue the channel
#define CMD_LATCH_OUT      0x00A9   // Transfer latch to output
#define CMD_LATCH_IN       0x00AA   // Transfer input to latch



// Commands for mdd8_send_channel_control

#define CTRL_DEVICE        0x0060   // Control the device
#define CTRL_RW_PTR        0x0061   // Set the read/write pointer
#define CTRL_R_PTR         0x0062   // Set the read pointer
#define CTRL_W_PTR         0x0063   // Set the write pointer
#define CTRL_SET_CB_CHANNEL 0x0064  // Set Callback procedure

// Commands for mdd8_send_channel_control_block

#define CTRL_CAPTION       0x0090   // Set caption of channel
                                    // HINT: Do not use this 
                                    // command,use procedure
                                    // mddX_set_caption



// Info types for mdd8_get_channel_info

#define INFO_STATUS        0x0040   // Status of channel
#define INFO_DATA_TYPE     0x0042   // Data type to transfer
#define INFO_TRANSFER_MODE 0x0043   // Supported transfer mode
#define INFO_MINSIZE       0x0044   // Minimum data length
#define INFO_MAXSIZE       0x0045   // Maximum data langth
#define INFO_RANGE         0x0046   // Range
#define INFO_RESOLUTION    0x0047   // Resolution
#define INFO_DEVICE        0x0048   // Device information
#define INFO_RW_PTR        0x0050   // Position read/write pointer
#define INFO_R_PTR         0x0051   // Position read pointer
#define INFO_W_PTR         0x0052   // Position write pointer



// Info types for mdd8_get_channel_info_block

#define INFO_CAPTION       0x0080   // Read caption of channel
                                    // HINT: Do not use this 
                                    // command,use procedure
                                    // mddX_get_caption




// Infotext numbers (Low-Byte = Page, High-Byte = Line)

#define ITNO_DEVICE_NAME      0x0000   // Name of device
#define ITNO_DEVICE_REVISION  0x0100   // Revision of device
#define ITNO_FIRM_NAME        0x0001   // Name of customer
#define ITNO_AUTHOR_NAME      0x0101   // Author of the MDD



// Data type identifiers

#define DATA_VOID       0x00000001UL   // No transfer data
#define DATA_BIT        0x00000002UL   // 1 bit
#define DATA_BYTE       0x00000003UL   // Unsigned 8 bit
#define DATA_CHAR       0x00000004UL   // Signed 8 bit
#define DATA_WORD       0x00000005UL   // Unsigned 16 bit
#define DATA_SHORT      0x00000006UL   // Signed 16 bit
#define DATA_DWORD      0x00000007UL   // Unsigned 32 bit
#define DATA_LONG       0x00000008UL   // Signed 32 bit
#define DATA_BLOCK      0x00000009UL   // Block (n bytes)
#define DATA_STREAM     0x0000000AUL   // Stream



// Data transfer modes (from command INFO_TRANSFER_MODE)

#define R_MODE          0x00000001UL   // Read only
#define W_MODE          0x00000002UL   // Write only
#define RW_MODE         0x00000003UL   // Read and write



// Physical I/O ranges (from command INFO_RANGE)

#define RNG_VOID           0x00000001UL   // No range
#define RNG_UPP_312MV5     0x00000021UL   // + 312.5 mV
#define RNG_UPP_625MV      0x00000022UL   // + 625 mV
#define RNG_UPP_1V25       0x00000023UL   // + 1.25 V
#define RNG_UPP_2V5        0x00000024UL   // + 2.5 V
#define RNG_UPP_5V         0x00000025UL   // + 5 V
#define RNG_UPP_10V        0x00000026UL   // + 10 V
#define RNG_UPN_312MV5     0x00000041UL   // - 312.5 mV
#define RNG_UPN_625MV      0x00000042UL   // - 625 mV
#define RNG_UPN_1V25       0x00000043UL   // - 1.25 V
#define RNG_UPN_2V5        0x00000044UL   // - 2.5 V
#define RNG_UPN_5V         0x00000045UL   // - 5 V
#define RNG_UPN_10V        0x00000046UL   // - 10 V
#define RNG_BIP_312MV5     0x00000061UL   // +/- 312.5 mV
#define RNG_BIP_625MV      0x00000062UL   // +/- 625 mV
#define RNG_BIP_1V25       0x00000063UL   // +/- 1.25 V
#define RNG_BIP_2V5        0x00000064UL   // +/- 2.5 V
#define RNG_BIP_5V         0x00000065UL   // +/- 5 V
#define RNG_BIP_10V        0x00000066UL   // +/- 10 V
#define RNG_20MA           0x00000081UL   // 20 mA


// Flags for channel status (from command INFO_STATUS)

#define _CSR_EXCLUSIVE     0x00000001UL   // Exclusive used
#define _CSR_HW_FORMAT     0x00000002UL   // Data in hardware format
#define _CSR_UNCORRECTED   0x00000004UL   // Uncorrected data
#define _CSR_OUT_SUSP      0x00010000UL   // Output suspended
#define _CSR_IN_SUSP       0x00020000UL   // Input suspended


// Device types

#define DEVICE_DIN         0x0101   // Digital input
#define DEVICE_DOUT        0x0102   // Digital output
#define DEVICE_DIO         0x0103   // Digital input/output
#define DEVICE_AIN_SE      0x0201   // Analog input, single-ended
#define DEVICE_AIN_DIFF    0x0202   // Analog input, difference
#define DEVICE_AOUT        0x0203   // Analog output
#define DEVICE_TRIG        0x0301   // Trigger device
#define DEVICE_CTRL        0x0401   // Control device
#define DEVICE_LED         0x0402   // LED
#define DEVICE_TIMER       0x0501   // Timer
#define DEVICE_WATCHDOG    0x0503   // Watchdog
#define DEVICE_COUNTER     0x0601   // Counter
#define DEVICE_CLOCK       0x0701   // Clock
#define DEVICE_CALENDAR    0x0702   // Calendar
#define DEVICE_SCC         0x0801   // Serial com. device
#define DEVICE_SQRWAVE     0x0901   // Square wave output
#define DEVICE_EEPROM      0x0A01   // Eeprom
#define DEVICE_HW_INT      0x0B01   // Hardware interrupt
#define DEVICE_SSI         0x0C01   // SSI device


// Physical I/O ranges

#define RANGE_VOID         0x0001   // No range
#define RANGE_UPP_312MV5   0x0021   // + 312.5 mV
#define RANGE_UPP_625MV    0x0022   // + 625 mV
#define RANGE_UPP_1V25     0x0023   // + 1.25 V
#define RANGE_UPP_2V5      0x0024   // + 2.5 V
#define RANGE_UPP_5V       0x0025   // + 5 V
#define RANGE_UPP_10V      0x0026   // + 10 V
#define RANGE_UPN_312MV5   0x0041   // - 312.5 mV
#define RANGE_UPN_625MV    0x0042   // - 625 mV
#define RANGE_UPN_1V25     0x0043   // - 1.25 V
#define RANGE_UPN_2V5      0x0044   // - 2.5 V
#define RANGE_UPN_5V       0x0045   // - 5 V
#define RANGE_UPN_10V      0x0046   // - 10 V
#define RANGE_BIP_312MV5   0x0061   // +/- 312.5 mV
#define RANGE_BIP_625MV    0x0062   // +/- 625 mV
#define RANGE_BIP_1V25     0x0063   // +/- 1.25 V
#define RANGE_BIP_2V5      0x0064   // +/- 2.5 V
#define RANGE_BIP_5V       0x0065   // +/- 5 V
#define RANGE_BIP_10V      0x0066   // +/- 10 V
#define RANGE_20MA         0x0081   // 20 mA


// Flags in CPS

#define _CP_NO_FLAGS       0x0000   // No flags
#define _CP_EXCLUSIVE      0x0001   // Use channel exclusive
#define _CP_HW_FORMAT      0x0002   // Use hardware format
#define _CP_UNCORRECTED    0x0004   // No use of correction

// flags for the 'mode' parameter in mdd_open_channel_callback

#define CBP_ASYNC_CALL					0x0002	// asynchronous callback without buffer (RT-only)
#define CBP_ON_MDD_MACHINE				0x0001   // if callprocedure is on the same cpu


// Read/Write modes in CPS

#define IO_MODE_DIRECT     0x0001   // Direct I/O
#define IO_MODE_LATCH      0x0002   // Latched I/O
#define IO_MODE_RAM        0x0004   // I/O to/from RAM
#define IO_MODE_RAM_LATCH  0x0006   // I/O to/from RAM/Latch



// Commands for mdd8_scan_channel

#define SCAN_SERVICE_LIST  0x0001   // Get the service list
#define SCAN_CDT           0x0002   // Get the CDT header
#define SCAN_HANDLE        0x0003   // Get the handles
#define SCAN_CPS           0x0004   // Get a CPS of a channel
#define SCAN_PARAMETER     0x0005   // Get channel parameter



// Error codes
//
// ERR_MDD_HANDLE_INVALID: 
//       The handle is invalid
// ERR_MDD_WRONG_DATA_TYPE: 
//       The used data type can not be transferred by the channel
// ERR_MDD_DRV_SERVICE:
//       Error in MDD service, diagnosis available 
//       (see mddX_get_diagnosis)
// ERR_MDD_CHANNEL_SERVICE:
//       Error in channel service, diagnosis available
//       (see mddX_get_diagnosis)
// ERR_MDD_SERVICE_NOT_AVAILABLE:
//       Service is not available
// ERR_MDD_DRV_NOT_INSTALLED:
//       MDD is not installed
// ERR_MDD_UNKNOWN_ERROR:
//       Unknown MDD error
// ERR_MDD_WRONG_OSX:
//       OsX must be updated

#ifdef __RT__
// On-Board platforms
#define ERR_MDD_HANDLE_INVALID            0x40E0
#define ERR_MDD_WRONG_DATA_TYPE           0x41E0
#define ERR_MDD_DRV_SERVICE               0x42E0
#define ERR_MDD_CHANNEL_SERVICE           0x43E0
#define ERR_MDD_SERVICE_NOT_AVAILABLE     0x44E0
#define ERR_MDD_DRV_NOT_INSTALLED         0x45E0
#define ERR_MDD_UNKNOWN_ERROR             0x46E0
#else // __RT__
// PC platforms
#define ERR_MDD_WRONG_OSX                 0x01
#define ERR_MDD_HANDLE_INVALID            0x40
#define ERR_MDD_WRONG_DATA_TYPE           0x41
#define ERR_MDD_DRV_SERVICE               0x42
#define ERR_MDD_CHANNEL_SERVICE           0x43
#define ERR_MDD_SERVICE_NOT_AVAILABLE     0x44
#define ERR_MDD_DRV_NOT_INSTALLED         0x45
#define ERR_MDD_UNKNOWN_ERROR             0x46
#endif // __RT__



// Diagnosis words

#define DIAG_NO_ERROR                     0x0000
#define DIAG_CALL_WITHOUT_EFFECT          0x0001
#define DIAG_WRONG_PARAMETER              0x0100
#define DIAG_WRONG_PARAMETER_MAX          0x01FF
#define DIAG_WRONG_NUMBER_OF_INPUTS       0x0201
#define DIAG_ABOVE_NUMBER_OF_INPUTS       0x0202
#define DIAG_BELOW_NUMBER_OF_INPUTS       0x0203
#define DIAG_WRONG_NUMBER_OF_RETURNS      0x0211
#define DIAG_ABOVE_NUMBER_OF_RETURNS      0x0212
#define DIAG_BELOW_NUMBER_OF_RETURNS      0x0213
#define DIAG_FUNCTION_LOCKED              0x0221
#define DIAG_RANGE_CROSS_OVER             0x0321
#define DIAG_RANGE_CROSS_UNDER            0x0322
#define DIAG_DATA_IMPOSSIBLE              0x0323
#define DIAG_END_OF_DATA                  0x0324
#define DIAG_MISSING_NULL_TERMINATION     0x0331
#define DIAG_NOT_ENOUGH_MEMORY            0x0401
#define DIAG_BUFFER_EMPTY                 0x0411
#define DIAG_BUFFER_FULL                  0x0412
#define DIAG_HARDWARE_CONFLICT            0x0501
#define DIAG_TIMEOUT                      0x0502
#define DIAG_DATA_LOST                    0x0503
#define DIAG_DEVICE_NOT_AVAILABLE         0x0511
#define DIAG_DEVICE_ALREADY_IN_USE        0x0512
#define DIAG_CHANNEL_NOT_FOUND            0x0531
#define DIAG_INFOTEXT_NOT_AVAILABLE       0x0541
#define DIAG_NO_CAPTION_BUFFER            0x0551
#define DIAG_CPS_NOT_SUPPORTED            0x0561
#define DIAG_RTC_WRITE_ACCESS             0x0601
#define DIAG_RTC_READ_ACCESS              0x0602
#define DIAG_EEPROM_WRITE_ACCESS          0x0603
#define DIAG_EEPROM_READ_ACCESS           0x0604
#define DIAG_INVALID                      0xFFFF



// Device status

#define DSC_SHARED_MAX        0xFFF0   // Device is connected to the
                                       // maximum number of channels
#define DSC_IS_NOT_AVAILABLE  0xFFF1   // Device is not available
#define DSC_IS_LOCKED         0xFFF2   // Device is locked
#define DSC_IS_EXCLUSIVE      0xFFFF   // Device is connected to an
                                       // exclusive opened channel



// Constants for the Channel-Descriptor-Table (CDT)

#define CDT_HEADER_SIZE       60       // Size of the CDT header
#define CDT_ID_CHANNEL_VALID  0x55     // ID of a valid CDT


// Null handle (handle variable will be set with that value when the
// handle specifies a not opened channel after opening or closing the
// channel)

#define MDD_NULL_HANDLE                   0UL



// M-D40-2

#define _MD402_OUTPUT_ENABLE_0_7          0x01
#define _MD402_OUTPUT_ENABLE_8_15         0x02
#define _MD402_OUTPUT_ENABLE_16_23        0x04
#define _MD402_OUTPUT_ENABLE_24_31        0x08
#define _MD402_OUTPUT_ENABLE_32_39        0x10
#define _MD402_OUTPUT_DISABLE_0_7         0x00
#define _MD402_OUTPUT_DISABLE_8_15        0x00
#define _MD402_OUTPUT_DISABLE_16_23       0x00
#define _MD402_OUTPUT_DISABLE_24_31       0x00
#define _MD402_OUTPUT_DISABLE_32_39       0x00

#define _MD402_POS_TRIGGER                0x01
#define _MD402_NEG_TRIGGER                0x00
#define _MD402_EXT_TRIGGER                0x02
#define _MD402_SOFT_TRIGGER               0x00

#define _MD402_ENABLE_INT_0               0x01
#define _MD402_ENABLE_INT_1               0x02
#define _MD402_ENABLE_INT_2               0x04
#define _MD402_ENABLE_INT_3               0x08
#define _MD402_DISABLE_INT_0              0x00
#define _MD402_DISABLE_INT_1              0x00
#define _MD402_DISABLE_INT_2              0x00
#define _MD402_DISABLE_INT_3              0x00

#define MD402_OUTPUT_CTRL                 0
#define MD402_INT_CTRL                    1
#define MD402_TRIG_CTRL                   2
#define MD402_SAMPLE_TRIGGER              0


// M-OPT-1/A

#define _MOPT1A_CON_DIN_0_IRQ_E           0x01
#define _MOPT1A_CON_DIN_1_IRQ_B           0x02



// M-OPT-1/B

#define _MOPT1B_ENABLE_DOUT_0_15          0x01

#define _MOPT1B_WATCHDOG_ENABLE           0x02
#define _MOPT1B_CON_WATCHDOG_IRQ_B        0x01



// M-RU8-2

#define  MRU82_MODE_0                     0x00
#define  MRU82_MODE_1                     0x01
#define  MRU82_MODE_2                     0x02
#define  MRU82_MODE_3                     0x03



// M-SH12-8

#define MSH128_SW_EXT_DIG_TRIG            0x00
#define MSH128_SW_TIMER_A                 0x01
#define MSH128_TIMER_A_POS_EXT_DIG_TRIG   0x02
#define MSH128_TIMER_A_POS_TIMER_A        0x03
#define MSH128_TIMER_A_NEG_EXT_DIG_TRIG   0x04
#define MSH128_TIMER_A_NEG_TIMER_A        0x05
#define MSH128_EXT_DIG_POS_EXT_DIG_TRIG   0x06
#define MSH128_EXT_DIG_NEG_EXT_DIG_TRIG   0x07
#define MSH128_SW_EXT_AN_TRIG             0x08
#define MSH128_TIMER_A_POS_EXT_AN_TRIG    0x0A
#define MSH128_TIMER_A_NEG_EXT_AN_TRIG    0x0C
#define MSH128_EXT_AN_TRIG_A_EXT_AN_TRIG  0x0E
#define MSH128_EXT_AN_TRIG_B_EXT_AN_TRIG  0x0F

#define MSH128_NONE                       0x00
#define MSH128_IRQ_B                      0x05
#define MSH128_IRQ_C                      0x06
#define MSH128_IRQ_D                      0x07



// M-DC15-2

#define _MDC152_LED_0_ON                  0x01
#define _MDC152_LED_1_ON                  0x02
#define _MDC152_LED_2_ON                  0x04
#define _MDC152_LED_3_ON                  0x08
#define _MDC152_LED_4_ON                  0x10
#define _MDC152_LED_5_ON                  0x20
#define _MDC152_LED_6_ON                  0x40
#define _MDC152_LED_7_ON                  0x80

#define _MDC152_CON_IP_0_IRQ_B            0x01
#define _MDC152_CON_IP_1_IRQ_E            0x02
#define _MDC152_CON_IP_2_IRQ_C            0x04
#define _MDC152_CON_IP_3_IRQ_D            0x08



// M-C16-3

#define _MC163_NO_IRQ                     0x00
#define _MC163_IRQ_A                      0x01
#define _MC163_IRQ_B                      0x02
#define _MC163_IRQ_C                      0x03
#define _MC163_IRQ_D                      0x04
#define _MC163_IRQ_E                      0x05
#define _MC163_IRQ_F                      0x06
#define _MC163_NMI                        0x07
#define _MC163_IRQ_POS_EDGE               0x00
#define _MC163_IRQ_NEG_EDGE               0x10

#define _MC163_DEMASK_INT_0               0x00000001UL
#define _MC163_DEMASK_INT_1               0x00000002UL
#define _MC163_DEMASK_INT_2               0x00000004UL
#define _MC163_DEMASK_INT_3               0x00000008UL
#define _MC163_DEMASK_INT_4               0x00000010UL
#define _MC163_DEMASK_INT_5               0x00000020UL
#define _MC163_DEMASK_INT_6               0x00000040UL
#define _MC163_DEMASK_INT_7               0x00000080UL
#define _MC163_DEMASK_INT_8               0x00000100UL
#define _MC163_DEMASK_INT_9               0x00000200UL
#define _MC163_DEMASK_INT_10              0x00000400UL
#define _MC163_DEMASK_INT_11              0x00000800UL
#define _MC163_DEMASK_INT_12              0x00001000UL
#define _MC163_DEMASK_INT_13              0x00002000UL
#define _MC163_DEMASK_INT_14              0x00004000UL
#define _MC163_DEMASK_INT_15              0x00008000UL

#define _MC163_INT_0_NEG_EDGE             0x00010000UL
#define _MC163_INT_1_NEG_EDGE             0x00020000UL
#define _MC163_INT_2_NEG_EDGE             0x00040000UL
#define _MC163_INT_3_NEG_EDGE             0x00080000UL
#define _MC163_INT_4_NEG_EDGE             0x00100000UL
#define _MC163_INT_5_NEG_EDGE             0x00200000UL
#define _MC163_INT_6_NEG_EDGE             0x00400000UL
#define _MC163_INT_7_NEG_EDGE             0x00800000UL
#define _MC163_INT_8_NEG_EDGE             0x01000000UL
#define _MC163_INT_9_NEG_EDGE             0x02000000UL
#define _MC163_INT_10_NEG_EDGE            0x04000000UL
#define _MC163_INT_11_NEG_EDGE            0x08000000UL

#define _MC163_LATCH_POS_EDGE             0x00
#define _MC163_LATCH_NEG_EDGE             0x20
#define _MC163_LATCH_DISABLE              0x00
#define _MC163_LATCH_ENABLE               0x40

#define _MC163_DIN_0_LATCH                0x00
#define _MC163_DIN_1_LATCH                0x01
#define _MC163_DIN_2_LATCH                0x02
#define _MC163_DIN_3_LATCH                0x03
#define _MC163_DIN_4_LATCH                0x04
#define _MC163_DIN_5_LATCH                0x05
#define _MC163_DIN_6_LATCH                0x06
#define _MC163_DIN_7_LATCH                0x07
#define _MC163_DIN_8_LATCH                0x08
#define _MC163_DIN_9_LATCH                0x09
#define _MC163_DIN_10_LATCH               0x0A
#define _MC163_DIN_11_LATCH               0x0B

#define _MC163_INACTIVE                   0x0000
#define _MC163_COUNTER_UP                 0x0001
#define _MC163_COUNTER_DN                 0x0002
#define _MC163_COUNTER_UP_DN_A            0x0003
#define _MC163_COUNTER_UP_DN_B            0x0004
#define _MC163_TIMER                      0x0005
#define _MC163_INCREMENT_A                0x0006
#define _MC163_INCREMENT_B                0x0007
#define _MC163_FREQUENCY                  0x0009
#define _MC163_PULSEWIDTH                 0x000A
#define _MC163_PERIOD                     0x000B
#define _MC163_VELOCITY                   0x000C
#define _MC163_MULTIPLE_PERIOD            0x1000

#define _MC163_AUTOSTART                  0x0020
#define _MC163_CONTINUE                   0x0040
#define _MC163_EXTERN                     0x0080
#define _MC163_SINGLE_A                   0x0000
#define _MC163_SINGLE_B                   0x0040
#define _MC163_DOUBLE_A                   0x0080
#define _MC163_DOUBLE_B                   0x00C0
#define _MC163_QUADRUPLE                  0x0100
#define _MC163_AUTOFREEZE                 0x0100
#define _MC163_AUTORESET                  0x0200
#define _MC163_AUTOLOAD                   0x0400
#define _MC163_OVERRIDE                   0x0800
#define _MC163_CASCADE_0_1                0x2000
#define _MC163_CASCADE_1_2                0x4000
#define _MC163_CASCADE_0_1_2              0x8000

#define _MC163_DIN_0_1_CNT_0              0x0000
#define _MC163_DIN_2_3_CNT_0              0x1000
#define _MC163_DIN_4_5_CNT_0              0x2000
#define _MC163_DIN_6_7_CNT_0              0x3000
#define _MC163_DIN_8_9_CNT_0              0x4000
#define _MC163_DIN_10_11_CNT_0            0x5000
#define _MC163_DIN_6_CNT_1                0x6000
#define _MC163_CNT_0_CNT_1                0x7000
#define _MC163_DIN_4_CNT_2                0x8000
#define _MC163_CNT_0_CNT_2                0x9000

#define _MC163_DIN_0_INV                  0x0001
#define _MC163_DIN_1_INV                  0x0002
#define _MC163_DIN_2_INV                  0x0004
#define _MC163_DIN_3_INV                  0x0008
#define _MC163_DIN_4_INV                  0x0010
#define _MC163_DIN_5_INV                  0x0020
#define _MC163_DIN_6_INV                  0x0040
#define _MC163_DIN_7_INV                  0x0080
#define _MC163_DIN_8_INV                  0x0100
#define _MC163_DIN_9_INV                  0x0200
#define _MC163_DIN_10_INV                 0x0400
#define _MC163_DIN_11_INV                 0x0800

#define _MC163_DOUT_0_CNT_0               0x0001
#define _MC163_DOUT_1_CNT_1               0x0002
#define _MC163_DOUT_2_CNT_2               0x0004
#define _MC163_DOUT_0_DIRECT              0x0000
#define _MC163_DOUT_1_DIRECT              0x0000
#define _MC163_DOUT_2_DIRECT              0x0000

#define _MC163_NO_START                   0x0000
#define _MC163_START_DIN_0                0x0010
#define _MC163_START_DIN_1                0x0020
#define _MC163_START_DIN_2                0x0030

#define _MC163_NO_STOP                    0x0000
#define _MC163_STOP_DIN_1                 0x0040
#define _MC163_STOP_DIN_2                 0x0080
#define _MC163_STOP_DIN_3                 0x00C0
#define _MC163_NO_STOP_UFL                0x0000
#define _MC163_STOP_UFL_CNT_0             0x0100
#define _MC163_STOP_UFL_CNT_1             0x0200
#define _MC163_STOP_UFL_CNT_2             0x0300

#define _MC163_NO_LOAD                    0x0000
#define _MC163_LOAD_DIN_0                 0x0400
#define _MC163_LOAD_DIN_1                 0x0800
#define _MC163_LOAD_DIN_2                 0x0C00

#define _MC163_NO_FREEZE                  0x0000
#define _MC163_FREEZE_DIN_1               0x1000
#define _MC163_FREEZE_DIN_2               0x2000
#define _MC163_FREEZE_DIN_3               0x3000

#define _MC163_NO_RESET                   0x0000
#define _MC163_RESET_DIN_0                0x4000
#define _MC163_RESET_DIN_1                0x8000
#define _MC163_RESET_DIN_2                0xC000

#define _MC163_NO_INT                     0x0000
#define _MC163_INT_CNT_0_UFL              0x0001
#define _MC163_INT_CNT_0_OVL              0x0002
#define _MC163_INT_CNT_0_END              0x0004
#define _MC163_INT_CNT_0_INVALID          0x0008
#define _MC163_INT_CNT_1_UFL              0x0010
#define _MC163_INT_CNT_1_OVL              0x0020
#define _MC163_INT_CNT_1_END              0x0040
#define _MC163_INT_CNT_2_UFL              0x0080
#define _MC163_INT_CNT_2_OVL              0x0100
#define _MC163_INT_CNT_2_END              0x0200

#define _MC163_NO_FREQ                    0x0000
#define _MC163_10_MHZ                     0x0002
#define _MC163_5_MHZ                      0x0003
#define _MC163_2_MHZ                      0x0004
#define _MC163_1_MHZ                      0x0005
#define _MC163_500_KHZ                    0x0006
#define _MC163_200_KHZ                    0x0007
#define _MC163_100_KHZ                    0x0008
#define _MC163_50_KHZ                     0x0009
#define _MC163_20_KHZ                     0x000A
#define _MC163_10_KHZ                     0x000B
#define _MC163_5_KHZ                      0x000C
#define _MC163_2_KHZ                      0x000D
#define _MC163_1_KHZ                      0x000E
#define _MC163_500_HZ                     0x000F
#define _MC163_200_HZ                     0x0010
#define _MC163_100_HZ                     0x0011
#define _MC163_50_HZ                      0x0012
#define _MC163_20_HZ                      0x0013
#define _MC163_10_HZ                      0x0014

#define _MC163_NO_PERIOD                  0x0000
#define _MC163_NO_TIME                    0x0000
#define _MC163_100_NS                     0x0002
#define _MC163_200_NS                     0x0003
#define _MC163_500_NS                     0x0004
#define _MC163_1_US                       0x0005
#define _MC163_2_US                       0x0006
#define _MC163_5_US                       0x0007
#define _MC163_10_US                      0x0008
#define _MC163_20_US                      0x0009
#define _MC163_50_US                      0x000A
#define _MC163_100_US                     0x000B
#define _MC163_200_US                     0x000C
#define _MC163_500_US                     0x000D
#define _MC163_1_MS                       0x000E
#define _MC163_2_MS                       0x000F
#define _MC163_5_MS                       0x0010
#define _MC163_10_MS                      0x0011
#define _MC163_20_MS                      0x0012
#define _MC163_50_MS                      0x0013
#define _MC163_100_MS                     0x0014

#define _MC163_CMD_START_COUNTER          0x01
#define _MC163_CMD_STOP_COUNTER           0x02
#define _MC163_CMD_LOAD_COUNTER           0x04
#define _MC163_CMD_FREEZE_COUNTER         0x08
#define _MC163_CMD_RESET_COUNTER          0x10

#define _MC163_COUNTER_0                  0x20
#define _MC163_COUNTER_1                  0x40
#define _MC163_COUNTER_2                  0x80

#define _MC163_NO_LED                     0x00
#define _MC163_LED_0                      0x01
#define _MC163_LED_1                      0x02



// 5Bx64

#define FBX64_SAMPLE_TRIGGER              0x0000
#define FBX64_WD_TRIGGER                  0x0001
#define FBX64_WD_STATUS                   0x0000

#define FBX64_STATUS_OK                   0x0000
#define FBX64_STATUS_WARNING              0x0001
#define FBX64_STATUS_ERROR                0x0101



// M-AX-32/SSI

#define _MAX32SSI_BINARY                  0x0000
#define _MAX32SSI_GRAYCODE                0x0080

#define _MAX32SSI_CLK312KHZ50             0x0001
#define _MAX32SSI_CLK208KHZ33             0x0002
#define _MAX32SSI_CLK156KHZ25             0x0003
#define _MAX32SSI_CLK125KHZ00             0x0004
#define _MAX32SSI_CLK104KHZ17             0x0005
#define _MAX32SSI_CLK89KHZ29              0x0006
#define _MAX32SSI_CLK78KHZ13              0x0007
#define _MAX32SSI_CLK69KHZ44              0x0008
#define _MAX32SSI_CLK62KHZ50              0x0009
#define _MAX32SSI_CLK56KHZ81              0x000A
#define _MAX32SSI_CLK52KHZ08              0x000B
#define _MAX32SSI_CLK48KHZ08              0x000C
#define _MAX32SSI_CLK44KHZ64              0x000D
#define _MAX32SSI_CLK41KHZ67              0x000E
#define _MAX32SSI_CLK39KHZ06              0x000F



// ML8 (Base Board)

#define ML8_EXTERNAL_LED                  0x0000
#define ML8_LOCAL_LED                     0x0001

#define ML8_TIMER_A                       0x0000
#define ML8_TIMER_B                       0x0001  
#define ML8_TIMER_C                       0x0002
#define ML8_TIMER_D                       0x0003
#define ML8_TIMER_E                       0x0004
#define ML8_TIMER_F                       0x0005

#define ML8_SQRWAVE_A                     0x0000
#define ML8_SQRWAVE_B                     0x0001
#define ML8_SQRWAVE_C                     0x0002

#define ML8_WATCHDOG                      0x0000
#define ML8_WATCHDOG_A                    0x0001
#define ML8_WATCHDOG_B                    0x0002
#define ML8_WATCHDOG_C                    0x0003

#define ML8_SCC_A                         0x0000
#define ML8_SCC_B                         0x0001

#define ML8_INT_NMI                       0x0000
#define ML8_INT_IRQ_A                     0x0001
#define ML8_INT_IRQ_B                     0x0002
#define ML8_INT_IRQ_C                     0x0003
#define ML8_INT_IRQ_D                     0x0004
#define ML8_INT_IRQ_E                     0x0005
#define ML8_INT_IRQ_F                     0x0006
#define ML8_INT_IRQ_G                     0x0007
#define ML8_INT_IRQ_H                     0x0008
#define ML8_INT_TIMER_A                   0x0009
#define ML8_INT_TIMER_B                   0x000A
#define ML8_INT_TIMER_C                   0x000B
#define ML8_INT_TIMER_D                   0x000C
#define ML8_INT_TIMER_E                   0x000D
#define ML8_INT_TIMER_F                   0x000E
#define ML8_INT_SCC_A                     0x000F
#define ML8_INT_SCC_B                     0x0010

#define ML8_UNMASK_INT                    0x00000000UL
#define ML8_MASK_INT                      0x00000001UL
#define ML8_POS_INT_EDGE                  0x00000002UL
#define ML8_NEG_INT_EDGE                  0x00000003UL
#define ML8_CLR_INT                       0x00000004UL
#define ML8_END_INT                       0x00000005UL

#define ML8_SET_RTS                       0x00000000UL
#define ML8_CLR_RTS                       0x00000001UL
#define ML8_SET_DTR                       0x00000002UL
#define ML8_CLR_DTR                       0x00000003UL

#define _ML8_COM_110                      0x0000
#define _ML8_COM_150                      0x0020
#define _ML8_COM_300                      0x0040
#define _ML8_COM_600                      0x0060
#define _ML8_COM_1200                     0x0080
#define _ML8_COM_2400                     0x00A0
#define _ML8_COM_4800                     0x00C0
#define _ML8_COM_9600                     0x00E0
#define _ML8_COM_19200                    0x0100
#define _ML8_COM_38400                    0x0200

#define _ML8_COM_CHR7                     0x0002
#define _ML8_COM_CHR8                     0x0003

#define _ML8_COM_STOP1                    0x0000
#define _ML8_COM_STOP2                    0x0004

#define _ML8_COM_NO_PARITY                0x0000
#define _ML8_COM_ODD_PARITY               0x0008
#define _ML8_COM_EVEN_PARITY              0x0018



// ML2 (Multi-LAB/2 board)

#define ML2D_LATCH_SOFTWARE               0
#define ML2D_LATCH_IP_0_POS               2
#define ML2D_LATCH_IP_0_NEG               3

#define ML2D_OUTPUT_DISABLE               0
#define ML2D_OUTPUT_ENABLE                1

#define ML2D_START_SOFTWARE               0
#define ML2D_START_IP_0_EDGE_POS          3
#define ML2D_START_IP_0_EDGE_NEG          2

#define _ML2D_COUNTER_EVENT               0
#define _ML2D_COUNTER_FREQ                1
#define _ML2D_COUNTER_PULSE               2
#define _ML2D_COUNTER_PERIOD              3
#define _ML2D_COUNTER_INCR_1              5
#define _ML2D_COUNTER_INCR_2              6
#define _ML2D_COUNTER_INCR_4              7

#define _ML2D_COUNTER_EDGE_POS            0x0000
#define _ML2D_COUNTER_EDGE_NEG            0x0008

#define _ML2D_COUNTER_START               0x0040

#define _ML2D_COUNTER_INPUT_8             0x0000
#define _ML2D_COUNTER_INPUT_9             0x0100
#define _ML2D_COUNTER_INPUT_10            0x0200
#define _ML2D_COUNTER_INPUT_11            0x0300
#define _ML2D_COUNTER_INPUT_12            0x0400
#define _ML2D_COUNTER_INPUT_13            0x0500
#define _ML2D_COUNTER_INPUT_14            0x0600
#define _ML2D_COUNTER_INPUT_15            0x0700

#define _ML2D_COUNTER_32BIT               0x8000

#define _ML2D_COUNTER_REF_TIMER_A         0x0000
#define _ML2D_COUNTER_REF_IP_2            0x0010

#define _ML2D_COUNTER_RESET_POS           0x0000
#define _ML2D_COUNTER_RESET_NEG           0x0100
#define _ML2D_COUNTER_LATCH_POS           0x0000
#define _ML2D_COUNTER_LATCH_NEG           0x0200




/********************************************************************
*  Channel property structures (CPS)
********************************************************************/



// M-D40-2
typedef struct _CPS_MD402
{
   USHORT   usDevice;
   USHORT   usIndexFirst;
   USHORT   usIndexLast;
   USHORT   usFlags;
   USHORT   usReadMode;
   USHORT   usWriteMode;
} CPS_MD402, *PCPS_MD402;



// M-OPT-1/A
typedef struct _CPS_MOPT1A
{
   USHORT   usDevice;
   USHORT   usIndexFirst;
   USHORT   usIndexLast;
   USHORT   usFlags;
   USHORT   usReadMode;
   USHORT   usWriteMode;
} CPS_MOPT1A, *PCPS_MOPT1A;



// M-OPT-1/B
typedef struct _CPS_MOPT1B
{
   USHORT   usDevice;
   USHORT   usIndexFirst;
   USHORT   usIndexLast;
   USHORT   usFlags;
   USHORT   usReadMode;
   USHORT   usWriteMode;
} CPS_MOPT1B, *PCPS_MOPT1B;



// M-RU8-2
typedef struct _CPS_MRU82
{
   USHORT   usDevice;
   USHORT   usIndexFirst;
   USHORT   usIndexLast;
   USHORT   usFlags;
   USHORT   usReadMode;
   USHORT   usWriteMode;
} CPS_MRU82, *PCPS_MRU82;



// M-DA16-2
typedef struct _CPS_MDA162
{
   USHORT   usDevice;
   USHORT   usIndexFirst;
   USHORT   usIndexLast;
   USHORT   usFlags;
   USHORT   usReadMode;
   USHORT   usWriteMode;
} CPS_MDA162, *PCPS_MDA162;



// M-DA2-2
typedef struct _CPS_MDA22
{
   USHORT   usDevice;
   USHORT   usIndexFirst;
   USHORT   usIndexLast;
   USHORT   usFlags;
   USHORT   usReadMode;
   USHORT   usWriteMode;
} CPS_MDA22, *PCPS_MDA22;



// M-DA4-2
typedef struct _CPS_MDA42
{
   USHORT   usDevice;
   USHORT   usIndexFirst;
   USHORT   usIndexLast;
   USHORT   usFlags;
   USHORT   usReadMode;
   USHORT   usWriteMode;
} CPS_MDA42, *PCPS_MDA42;



// M-AD16-4
typedef struct _CPS_MAD164
{
   USHORT   usDevice;
   USHORT   usIndexFirst;
   USHORT   usIndexLast;
   USHORT   usFlags;
   USHORT   usReadMode;
   USHORT   usOverSampling;
} CPS_MAD164, *PCPS_MAD164;



// M-AD16-3
typedef struct _CPS_MAD163
{
   USHORT   usDevice;
   USHORT   usIndexFirst;
   USHORT   usIndexLast;
   USHORT   usFlags;
   USHORT   usReadMode;
   USHORT   usRange;
   USHORT   usSettleTime;
} CPS_MAD163, *PCPS_MAD163;



// M-AD12-16
typedef struct _CPS_MAD1216
{
   USHORT   usDevice;
   USHORT   usIndexFirst;
   USHORT   usIndexLast;
   USHORT   usFlags;
   USHORT   usReadMode;
   USHORT   usRange;
} CPS_MAD1216, *PCPS_MAD1216;



// M-SH12-8
typedef struct _CPS_MSH128
{
   USHORT   usDevice;
   USHORT   usIndexFirst;
   USHORT   usIndexLast;
   USHORT   usFlags;
   USHORT   usReadMode;
   USHORT   usWriteMode;
} CPS_MSH128, *PCPS_MSH128;



// M-5B-1/U
typedef struct _CPS_M5B1U
{
   USHORT   usDevice;
   USHORT   usIndexFirst;
   USHORT   usIndexLast;
   USHORT   usFlags;
   USHORT   usReadMode;
   USHORT   usWriteMode;
   USHORT   usSettleTime;
   USHORT   usRange;
} CPS_M5B1U, *PCPS_M5B1U;



// 5B panels
typedef struct _CPS_M5BP
{
   USHORT   usDevice;
   USHORT   usIndexFirst;
   USHORT   usIndexLast;
   USHORT   usFlags;
   USHORT   usReadMode;
   USHORT   usWriteMode;
   USHORT   usSettleTime;
   USHORT   usRange;
} CPS_M5BP, *PCPS_M5BP;



// M-DC15-2
typedef struct _CPS_MDC152
{
   USHORT   usDevice;
   USHORT   usIndexFirst;
   USHORT   usIndexLast;
   USHORT   usFlags;
   USHORT   usReadMode;
   USHORT   usWriteMode;
} CPS_MDC152, *PCPS_MDC152;



// M-C16-3
typedef struct _CPS_MC163
{
   USHORT   usDevice;
   USHORT   usIndexFirst;
   USHORT   usIndexLast;
   USHORT   usFlags;
   USHORT   usReadMode;
	USHORT   usWriteMode;
   USHORT   usMode;
   USHORT   usCounterInput;
   USHORT   usCounterOutput;
   USHORT   usCommandInput;
   USHORT   usInterrupt;
   USHORT   usTime;
   USHORT   usMultiple;
} CPS_MC163, *PCPS_MC163;



// 5BX64 Panels
typedef struct _CPS_5BX64
{
   USHORT   usDevice;
   USHORT   usIndexFirst;
   USHORT   usIndexLast;
   USHORT   usIndexExtern;
   USHORT   usFlags;
   USHORT   usReadMode;
   USHORT   usWriteMode;
} CPS_5BX64, *PCPS_5BX64;



// M-AX-32/SSI
typedef struct _CPS_MAX32SSI
{
   USHORT   usDevice;
   USHORT	usIndexFirst;
   USHORT	usIndexLast;
   USHORT	usFlags;
	USHORT	usReadMode;
	USHORT	usMode;
	USHORT	usDataBits;
} CPS_MAX32SSI, *PCPS_MAX32SSI;



// Base board (ML8)
typedef struct _CPS_ML8
{
   USHORT   usDevice;
   USHORT   usIndexFirst;
   USHORT   usIndexLast;
   USHORT   usFlags;
   USHORT   usReadMode;
   USHORT   usWriteMode;
   ULONG    ulTimeOut;
   USHORT   usMode;
} CPS_ML8, *PCPS_ML8;



// Base board (ML6)
typedef struct _CPS_ML6
{
   USHORT   usDevice;
   USHORT   usIndexFirst;
   USHORT   usIndexLast;
   USHORT   usFlags;
   USHORT   usReadMode;
   USHORT   usWriteMode;
   ULONG    ulTimeOut;
   USHORT   usMode;
} CPS_ML6, *PCPS_ML6;



// Multi-LAB/2
typedef struct _CPS_ML2
{
   USHORT   usDevice;
   USHORT   usIndexFirst;
   USHORT   usIndexLast;
   USHORT   usFlags;
   USHORT   usReadMode;
   USHORT   usWriteMode;
   USHORT   usRange;
   USHORT   usMode;
} CPS_ML2, *PCPS_ML2;

/********************************************************************
*  Header of the Channel-Descriptor-Table (CDT)
********************************************************************/

typedef struct
{
   ULONG    pfnServiceDataOut;
   ULONG    pfnServiceDataIn; 
   ULONG    pfnServiceSpecial;
   ULONG    pfnServiceRes1;   
   UCHAR    ucCdtID;          
   UCHAR    ucDataTypeID;     
   UCHAR    ucCdtType;        
   UCHAR    ucMddTaskNo;      
   USHORT   usDataSizeMin;    
   USHORT   usDataSizeMax;    
   UCHAR    ucPhysIORange;    
   UCHAR    ucResolution;     
   UCHAR    ucTransferMode;   
   UCHAR    ucServiceListNo;  
   ULONG    ulCSR;            
   ULONG    ulPreviousCDT;    
   ULONG    pfnServiceCopy1;  
   ULONG    pfnServiceCopy2;  
   ULONG    ulRes2;           
   USHORT   usCdtSize;        
   UCHAR    ucParSize;        
   UCHAR    ucCpsCopySize;    
   UCHAR    ucCaptionSize;    
   UCHAR    ucRes3;           
   USHORT   usRelOfsCpsCopy;  
   USHORT   usRelOfsCaption;  
   USHORT   usRes4;           
#ifdef LINUX	// there is a "#define MDDx_CDT_HEAD MDD8_CDT_HEAD" in MLxADAP.H, so for gcc this looks like a multiple definition of MDD8_CDT_HEAD
}  MDD8_CDT_HEAD;
#else
}  MDD8_CDT_HEAD, MDD6_CDT_HEAD, MDD2_CDT_HEAD;
#endif


#endif // !_INC_MDDCPS






